from decimal import Decimal
from django.conf import settings
from django.test import TestCase

from exchange_eclairs import RateFetchError, RateFetchClient

class RateFetchTests(TestCase):
	# Tests the RateFetchClient

	def setUp(self):
		self.fetcher = RateFetchClient(settings.OPENEXCHANGERATES_APP_ID)

	def tearDown(self):
		self.fetcher.fetcher.close()

	def test_initialization(self):
		# Test that the app_id parameter is added 
		# to the newly initialized :class:RateFetchClient
		self.assertTrue('app_id' in self.fetcher.fetcher.params)

	def test_query_rates(self):
		# Assert that we do indeed have a
		# JSON response from the API endpoint
		response = self.fetcher.query_rates()
		self.assertIsInstance(response, dict)
		self.assertTrue('USD' in response)

	def test_query_currency(self):
		# Assert that a dictionary
		# of currencies is returned
		response = self.fetcher.query_currency()
		self.assertIsInstance(response, dict)
		self.assertTrue('USD' in response)

	def test_exchange_currency(self):
		# Test that currencies are exchanged
		amnt = Decimal('15.00')
		amount = self.fetcher.exchange_currency(amnt, 'UsD', 'cAd', True)
		self.assertIsInstance(amount, tuple)

		amount = self.fetcher.exchange_currency(amnt, 'uSd', 'cad')
		self.assertIsInstance(amount, Decimal)

		self.assertRaises(Exception, lambda: self.fetcher.exchange_currency(amnt, 'bore', 'cad'))


import requests
from decimal import Decimal
from django.conf import settings

from exchange.exceptions import RateFetchError

if settings.OPENEXCHANGERATES_PAID_CX:
	BASE_URL = 'https://openexchangerates.org/api/'
else:
	BASE_URL = 'http://openexchangerates.org/api/'

LATEST_URL = BASE_URL + 'latest.json'
CURRENCY_URL = BASE_URL + 'currencies.json'

class RateFetchClient(object):
	"""
	This class creates an object for fetching
	and dealing with exchange rates.

	"""

	def __init__(self, key=None):
		"""
		Creates a new fetcher instance

		"""
		self.fetcher = requests.Session()
		if not key:
			key = settings.OPENEXCHANGERATES_APP_ID
		self.fetcher.params.update({'app_id': key})

	def query_rates(self, base=None):
		"""
		Fetches the rates JSON object from 
		the `OPENEXCHANGERATES_LATEST_URL` API gateway

		Refer to http://openexchangerates.org for more 
		information

		"""
		try:

			r = self.fetcher.get(LATEST_URL)
			
			if base and settings.OPENEXCHANGERATES_PAID_CX:
				r = self.fetcher.get(LATEST_URL, params={'base': base.upper()})
			r.raise_for_status()
			
			return r.json(parse_int=Decimal, parse_float=Decimal)['rates']

		except requests.exceptions.RequestException, error:
			raise RateFetchError(error)

	def restrict_rates(self, base=None, symbols=None):
		"""
		Makes a more restrictive query for 
		particular currencies.

		:param symbols:
			This is a list of currency codes for which 
			we would like exchange rates returned

		:param base:
			The base currency we would like for our 
			returned JSON. Take a look at 'http://openexchangerates.org'

		:return:
			Currency dictionary

		Note that only users signed up for the Ultimate or Enterprise
		plans may make use of this particular method

		"""
		param = {}
		if base:
			param['base'] = base.upper()

		if symbols:
			symbols = ','.join(symbols)
			param['symbols'] = symbols.upper()

		try:
			r = self.fetcher.get(LATEST_URL, params=param)
		except requests.exceptions.RequestException, error:
			raise RateFetchError(error)

		return r.json(parse_int=Decimal, parse_float=Decimal)['rates']

	def query_currency(self):
		"""
		Fetch a list of currencies

		"""
		try:
			r = self.fetcher.get(CURRENCY_URL)
			return r.json()
		except requests.exceptions.RequestException, error:
			raise RateFetchError(error)

	def exchange_currency(self, amount, _from, _to, rate=False):
		"""
		Using the base currency provided, this works on the 
		premise that if, for instance, 
		==> 1USD == 1.08CAD and 1USD == 0.73EUR, 
		==> then 1USD == 1.08CAD == 0.73EUR, 

		which means that 
		
		==> 1CAD == ((0.73 * 1) / 1.08) EUR. 

		Following this logic ...

		:param amount:
			The amount to be converted

		:param _from:
			The currency from which the exchange is to be made

		:param _to:
			The currency to which the exchange is to be made

		:param rate:
			If True, returns the conversion rate alongside 
			the converted amount

		:return:
			The converted amount intended in the "_to" currency
			(See above regarding the return of conversion rates)

		"""
		_from = _from.upper()
		_to = _to.upper()

		try:
			rates = self.query_rates()
		except requests.exceptions.RequestException, error:
			raise RateFetchError(error)

		if _from in rates and _to in rates:

			conv_rate = rates[_to] / rates[_from]
			amount = amount * conv_rate
			if rate:
				return (amount, conv_rate)
			else: return amount

		raise Exception('Unable to convert funds. Please try again.')



import os
from setuptools import setup, find_packages

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django_broker',
    version='0.0.4',
    packages=['exchange'],
    include_package_data=True,
    license='BSD License',
    description='A simple Django app to facilitate currency conversions.',
    long_description=README,
    url='http://github.com/parsenz/django-broker',
    author='Seiyifa Tawari',
    author_email='seiyifa.tawari@gmail.com',
    install_requires=['Django'],
    keywords='django exchange rates currency',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
    ],
)
